@@ -7,29 +7,25 @@ use Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(bgs_call bgs_back bgs_wait bgs_break);
 
-our $VERSION = '0.12';
+our $VERSION = '1.00';
 
 use IO::Select;
-use Scalar::Util qw(refaddr);
 use Storable qw(freeze thaw);
 use POSIX ":sys_wait_h";
 
 
 our $limit = 0; 
 
+$SIG{CHLD} = "IGNORE";
 
 my $sel = IO::Select->new();
+my %callbacks = (); 
 
-my %fh2data   = (); 
-my %vpid2data = (); 
-my @to_call   = (); 
+my @todo_call = (); 
 
 
-
-sub _call {
-	my ($data) = @_;
-
-	my $sub = delete $$data{sub};
+sub _bgs_call($$) {
+	my ($sub, $callback) = @_;
 
 	pipe my $from_kid_fh, my $to_parent_fh or die "pipe: $!";
 
@@ -38,139 +34,68 @@ sub _call {
 
 	if ($kid_pid) {
 		$sel->add($from_kid_fh);
-
-		my $vpid = $$data{vpid};
-
-		$$data{fh}  = $from_kid_fh;
-		$$data{pid} = $kid_pid;
-
-		$fh2data{$from_kid_fh} = $data;
-		$vpid2data{$vpid}      = $data;
-
+		$callbacks{$from_kid_fh} = $callback;
 	} else {
-		%fh2data   = ();
-		%vpid2data = ();
-		@to_call   = ();
-
 		binmode $to_parent_fh;
 		print $to_parent_fh freeze \ scalar $sub->();
 		close $to_parent_fh;
 		exit;
 	}
-
-}
-
-
-sub _bgs_call {
-	my ($sub, $callback) = @_;
-
-	my $data = { sub => $sub };
-	my $vpid = $$data{vpid} = refaddr $data;
-
-	$$data{callback} = $callback if $callback;
-
-	if ($limit > 0 and keys %fh2data >= $limit) {
-		push @to_call, $data;
-	} else {
-		_call($data);
-	}
-
-	return $data;
+	return $kid_pid;
 }
 
 sub bgs_call(&$) {
 	my ($sub, $callback) = @_;
-
-	my $data = _bgs_call($sub, $callback);
-
-	return $$data{vpid};
+	if ($limit > 0 and keys %callbacks >= $limit) {
+		push @todo_call, [$sub, $callback];
+	} else {
+		_bgs_call($sub, $callback);
+	}
 }
 
 sub bgs_back(&) { shift }
 
 
-sub bgs_wait(;$) {
-	my ($waited) = @_;
-
-	if ($waited and not exists $vpid2data{$waited} and not grep { $$_{vpid} eq $waited } @to_call) {
-		return;
-	}
-
+sub bgs_wait() {
 	local $SIG{PIPE} = "IGNORE";
+	my %from_kid;       
 	my $buf;            
 	my $blksize = 1024; 
-	while ($sel->count()) {
-		foreach my $fh ($sel->can_read()) {
-			my $data = $fh2data{$fh};
-			my $len = sysread $fh, $buf, $blksize;
-			if ($len) {
-				push @{$$data{from_kid}}, $buf; 
-			} elsif (defined $len) { 
+ 	while ($sel->count()) {
+ 		foreach my $fh ($sel->can_read()) {
+ 			my $len = sysread $fh, $buf, $blksize;
+ 			if ($len) {
+ 				push @{$from_kid{$fh}}, $buf;
+ 			} elsif (defined $len) { 
 				$sel->remove($fh); 
 				close $fh or warn "Kid is existed: $?";
 
-				delete $$data{fh};
-				my $pid = delete $$data{pid};
-				my $callback = delete $$data{callback};
-				
-				unless ($$data{break}) {
-					if (exists $$data{from_kid} and my $r = eval { thaw(join "", @{delete $$data{from_kid}}) }) {
-						if ($callback) {
-							$callback->($$r);
-						} else {
-							$$data{result} = $$r;
-						}
-					} else {
-						if ($callback) {
-							$callback->();
-						} else {
-							$$data{result} = undef;
-						}
-					}
+				if (exists $from_kid{$fh}) {
+	 				my $r = join "", @{$from_kid{$fh}};
+ 					delete $from_kid{$fh};
+					$callbacks{$fh}->(${thaw $r});
+				} else {
+					$callbacks{$fh}->();
 				}
 
-				my $vpid = $$data{vpid};
-				delete $fh2data{$fh};
-				delete $vpid2data{$vpid};
+ 				delete $callbacks{$fh};
 
-				waitpid($pid, 0);
-
-				if (my $call = shift @to_call) {
-					_call($call);
+				if (my $call = shift @todo_call) {
+					_bgs_call($$call[0], $$call[1]);
 				}
 
-				if ($waited and $waited == $vpid) {
-					return;
-				}
-
-			} else {
-				die "Can't read '$fh': $!";
-			}
-		}
-	}
+ 			} else {
+ 				die "Can't read '$fh': $!";
+ 			}
+ 		}
+ 	}
 }
 
 
-
-sub bgs_break(;$) {
-	my ($vpid) = @_;
-	if (defined $vpid) {
-		my $data = $vpid2data{$vpid};
-		defined $data or return;
-		if (my $pid = $$data{pid}) {
-			$$data{break} = 1;
-			kill "TERM", $pid;
-		}
-		 @to_call = grep { $$_{vpid} ne $vpid } @to_call;
-	} else {
-		foreach my $data (values %vpid2data) {
-			if (my $pid = $$data{pid}) {
-				$$data{break} = 1;
-				kill "TERM", $pid;
-			}
-		}
-		@to_call = ();
-	}
+sub bgs_break() {
+	local $SIG{TERM} = "IGNORE";
+	kill 15, -$$;
+	1 while waitpid(-1, WNOHANG) > 0;
 }
 
 
@@ -223,7 +148,7 @@ The subroutine must return either a B<scalar> or a B<reference>!
 The answer of the subroutine passes to the callback subroutine as an argument.
 If a child process ended without bgs_call value returning, than bgs_back subprogram is called without argument.
 
-bgs_call return vpid (virtual pid) of child process.
+bgs_call return PID of child proces.
 
 =head2 bgs_back
 
@@ -237,13 +162,9 @@ as an argument.
 Call of bgs_wait() reduces to child processes answers wait and
 callback subroutines execution.
 
-Call bgs_wait($vpid) to wait specific process.
-
 =head2 bgs_break
 
-kill all or specific child processes.
-
-Call bgs_break($vpid) to kill specific process.
+kill all child processes.
 
 =head2 $BGS::limit
 
